import 'package:flutter/foundation.dart';

extension ListExt<E> on List<E> {
  /// returns the element of this list that is next to [current].
  /// If current either the last element or is not in the list, returns the first element
  E nextElement(E current) => this[(indexOf(current) + 1) % length];

  /// Get enum entry from [name] or [fallback] if no entry matches given name.
  /// Set [ignoreCase] to true for non-case-sensitive name comparison
  /// Also check [requireFromName], [optFromName]
  /// E.g. CrossAxisAlignment.values.fromName('center', fallback: CrossAxisAlignment.start)
  E fromName(String? name, {required E fallback, bool ignoreCase = false}) =>
      optFromName(name, ignoreCase: ignoreCase) ?? fallback;

  /// Get enum entry from [name] or null if no entry matches given name.
  /// Set [ignoreCase] to true for non-case-sensitive name comparison
  /// Also check [fromName], [requireFromName]
  /// E.g. CrossAxisAlignment.values.optFromName('center')
  E? optFromName(String? name, {bool ignoreCase = false}) {
    if (name != null) {
      if (ignoreCase) {
        name = name.toLowerCase();
        for (final entry in this) if (entry != null && describeEnum(entry).toLowerCase() == name) return entry;
      } else {
        for (final entry in this) if (entry != null && describeEnum(entry) == name) return entry;
      }
    }
    return null;
  }

  /// Require enum entry from [name] or throw [ArgumentError] if no entry matches given name.
  /// Set [ignoreCase] to true for non-case-sensitive name comparison
  /// Also check [fromName], [optFromName]
  /// E.g. CrossAxisAlignment.values.requireFromName('center')
  E requireFromName(String? name, {bool ignoreCase = false}) {
    final entry = optFromName(name, ignoreCase: ignoreCase);
    if (entry != null) return entry;
    throw ArgumentError.value(name, 'name');
  }

  /// Update the first element of this list that satisfies [test] predicate with [newValue]
  bool updateWhere(bool Function(E element) test, E newValue) {
    for (var i = 0; i < length; i++) {
      if (test(this[i])) {
        this[i] = newValue;
        return true;
      }
    }
    return false;
  }

  /// Update all the elements of this list that satisfies [test] predicate with [update] function
  int updateAllWhere(bool Function(E element) test, E Function(int index, E oldValue) update) {
    var count = 0;
    for (var i = 0; i < length; i++) {
      final value = this[i];
      if (test(value)) {
        this[i] = update(i, value);
        count++;
      }
    }
    return count;
  }
}
