import 'package:flutter/cupertino.dart';
import 'package:flutterx_utils/src/extensions/list.dart';

extension MapExt<K, V> on Map<K, V> {
  /// Get enum entry from name at given [key] or [fallback] if no entry matches name/index at given key.
  /// Set [ignoreCase] to true for non-case-sensitive name comparison
  /// Set [index] to true to read [key] as index instead of name
  /// Also check [optEnum], [requireEnum]
  /// E.g. json.getEnum('alignment', CrossAxisAlignment.values, fallback: CrossAxisAlignment.start)
  E getEnum<E>(String key, List<E> values, {required E fallback, bool ignoreCase = false, bool index = false}) =>
      optEnum<E>(key, values, ignoreCase: ignoreCase, index: index) ?? fallback;

  /// Get enum entry from name at given [key] or null if no entry matches name/index at given key.
  /// Set [ignoreCase] to true for non-case-sensitive name comparison
  /// Set [index] to true to read [key] as index instead of name
  /// Also check [getEnum], [requireEnum]
  /// E.g. json.optEnum('alignment', CrossAxisAlignment.values)
  E? optEnum<E>(String key, List<E> values, {bool ignoreCase = false, bool index = false}) {
    if (index) {
      final i = this[key] as int;
      if (i < 0 || i >= values.length) return null;
      return values[i];
    }
    return values.optFromName(this[key]?.toString(), ignoreCase: ignoreCase);
  }

  /// Get all values where key satisfies [test] predicate
  Iterable<V> valuesWhereKey(bool Function(K key) test) sync* {
    for (final entry in entries) if (test(entry.key)) yield entry.value;
  }

  /// Get all keys where value satisfies [test] predicate
  Iterable<K> keysWhereValue(bool Function(V value) test) sync* {
    for (final entry in entries) if (test(entry.value)) yield entry.key;
  }

  /// Get enum entry from name at given [key] or throw [ArgumentError] if no entry matches name/index at given key.
  /// Set [ignoreCase] to true for non-case-sensitive name comparison
  /// Set [index] to true to read [key] as index instead of name
  /// Also check [getEnum], [optEnum]
  /// E.g. json.requireEnum('alignment', CrossAxisAlignment.values)
  E requireEnum<E>(String key, List<E> values, {bool ignoreCase = false, bool index = false}) {
    final entry = optEnum<E>(key, values, ignoreCase: ignoreCase, index: index);
    if (entry != null) return entry;
    throw ArgumentError.value(this[key], key);
  }

  /// Get entry value at given [key] or throw [ArgumentError] if value is not of type [E].
  /// E.g. json.require<int>('id') // throws if null
  E require<E>(String key) {
    final value = this[key];
    if (value is E) return value;
    throw ArgumentError.value(this[key], key);
  }

  /// Get parametrized list at given [key] from json map or [fallback] if specified or empty list of [T]
  /// throws a [StateError] if element at given key is not a list
  List<T> getList<T>(String key, {ValueGetter<List<T>>? fallback}) =>
      optList<T>(key) ?? fallback?.call() ?? List<T>.empty();

  /// Get parametrized list at given [key] from json map or null if value is null
  /// throws a [StateError] if element at given key is not a list
  List<T>? optList<T>(String key) {
    final list = this[key];
    if (list == null) return null;
    if (list is List<T>) return list;
    if (list is List) return list.cast<T>();
    throw StateError('Expected element of type List at $key, found ${list.runtimeType}');
  }

  /// Get parametrized list at given [key] from json map or throw [ArgumentError] if null
  /// throws a [StateError] if element at given key is not a list
  List<T> requireList<T>(String key) {
    final list = optList<T>(key);
    if (list != null) return list;
    throw ArgumentError.value(this[key], key);
  }
}
